@@ -44,7 +44,7 @@
 /**
  * Default memory allowed per connection.
  */
-#define MHD_POOL_SIZE_DEFAULT (1024 * 1024)
+#define MHD_POOL_SIZE_DEFAULT (32 * 1024)
 
 /**
  * Print extra messages with reasons for closing
@@ -62,7 +62,18 @@
 #ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
+#ifndef MSG_DONTWAIT
+#define MSG_DONTWAIT 0
 #endif
+#endif
+
+#ifdef __SYMBIAN32__
+static void pthread_kill (int, int) {
+  // Symbian doesn't have signals. The user of the library is required to
+  // run it in an external select loop.
+  abort();
+}
+#endif  // __SYMBIAN32__
 
 /**
  * Trace up to and return master daemon. If the supplied daemon
@@ -202,7 +213,7 @@ MHD_ip_limit_add(struct MHD_Daemon *daemon,
   MHD_ip_count_lock (daemon);
 
   /* Search for the IP address */
-  node = TSEARCH (key, &daemon->per_ip_connection_count, MHD_ip_addr_compare);
+  node = (void*)TSEARCH (key, &daemon->per_ip_connection_count, MHD_ip_addr_compare);
   if (!node)
     {
 #if HAVE_MESSAGES
@@ -254,7 +265,7 @@ MHD_ip_limit_del(struct MHD_Daemon *daemon,
   MHD_ip_count_lock (daemon);
 
   /* Search for the IP address */
-  node = TFIND (&search_key, &daemon->per_ip_connection_count, MHD_ip_addr_compare);
+  node = (void*)TFIND (&search_key, &daemon->per_ip_connection_count, MHD_ip_addr_compare);
 
   /* Something's wrong if we couldn't find an IP address
    * that was previously added */
@@ -466,18 +477,22 @@ MHD_handle_connection (void *data)
       now = time (NULL);
       tv.tv_usec = 0;
       if (timeout > (now - con->last_activity))
-        tv.tv_sec = timeout - (now - con->last_activity);
-      else
-        tv.tv_sec = 0;
-      if ((con->state == MHD_CONNECTION_NORMAL_BODY_UNREADY) ||
-          (con->state == MHD_CONNECTION_CHUNKED_BODY_UNREADY))
 	{
-          /* do not block (we're waiting for our callback to succeed) */
-	  timeout = 1;  
+	  /* in case we are missing the SIGALRM, keep going after
+	     at most 1s; see http://lists.gnu.org/archive/html/libmicrohttpd/2009-10/msg00013.html */
+	  tv.tv_sec = 1;
+	  if ((con->state == MHD_CONNECTION_NORMAL_BODY_UNREADY) ||
+	      (con->state == MHD_CONNECTION_CHUNKED_BODY_UNREADY))
+	    {
+	      /* do not block (we're waiting for our callback to succeed) */
+	      tv.tv_sec = 0;
+	    }	
+	}
+      else
+	{
 	  tv.tv_sec = 0;
 	}
-      num_ready = SELECT (max + 1,
-                          &rs, &ws, &es, (timeout != 0) ? &tv : NULL);
+      num_ready = SELECT (max + 1, &rs, &ws, &es, &tv);
       if (num_ready < 0)
         {
           if (errno == EINTR)
@@ -577,7 +592,6 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
   memset (addr, 0, sizeof (addrstorage));
 
   s = ACCEPT (daemon->socket_fd, addr, &addrlen);
-
   if ((s < 0) || (addrlen <= 0))
     {
 #if HAVE_MESSAGES
@@ -593,6 +607,19 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
         }
       return MHD_NO;
     }
+  if (s >= FD_SETSIZE)
+    {
+#if HAVE_MESSAGES
+      MHD_DLOG (daemon,
+		"Socket descriptor larger than FD_SETSIZE: %d > %d\n",
+		s,
+		FD_SETSIZE);
+#endif     
+      CLOSE (s);
+      return MHD_NO;
+    }
+
+
 #if HAVE_MESSAGES
 #if DEBUG_CONNECT
   MHD_DLOG (daemon, "Accepted connection on socket %d\n", s);
@@ -740,6 +767,7 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
   struct MHD_Connection *pos;
   struct MHD_Connection *prev;
   void *unused;
+  int rc;
 
   pos = daemon->connections;
   prev = NULL;
@@ -755,8 +783,15 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
             prev->next = pos->next;
           if (0 != (pos->daemon->options & MHD_USE_THREAD_PER_CONNECTION))
             {
-              pthread_kill (pos->pid, SIGALRM);
-              pthread_join (pos->pid, &unused);
+	      pthread_kill (pos->pid, SIGALRM);
+              if (0 != (rc = pthread_join (pos->pid, &unused)))
+		{
+#if HAVE_MESSAGES
+		  MHD_DLOG (daemon, "Failed to join a thread: %s\n",
+			    STRERROR (rc));
+#endif
+		  abort();		
+		}
             }
           MHD_destroy_response (pos->response);
           MHD_pool_destroy (pos->pool);
@@ -870,7 +905,11 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
         return MHD_NO;
       FD_SET (max, &rs);
     }
-
+  
+  /* in case we are missing the SIGALRM, keep going after
+     at most 1s; see http://lists.gnu.org/archive/html/libmicrohttpd/2009-10/msg00013.html */
+  timeout.tv_usec = 0;
+  timeout.tv_sec = 1; 
   if (may_block == MHD_NO)
     {
       timeout.tv_usec = 0;
@@ -879,16 +918,14 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
   else
     {
       /* ltimeout is in ms */
-      if (MHD_YES == MHD_get_timeout (daemon, &ltimeout))
-        {
-          timeout.tv_usec = (ltimeout % 1000) * 1000;
-          timeout.tv_sec = ltimeout / 1000;
-          may_block = MHD_NO;
+      if ( (MHD_YES == MHD_get_timeout (daemon, &ltimeout)) &&
+	   (ltimeout < 1000) )
+	{
+          timeout.tv_usec = ltimeout * 1000;
+          timeout.tv_sec = 0;
         }
     }
-
-  num_ready = select (max + 1, &rs, &ws, &es, may_block == MHD_NO ? &timeout
-                      : NULL);
+  num_ready = SELECT (max + 1, &rs, &ws, &es, &timeout);
 
   if (daemon->shutdown == MHD_YES)
     return MHD_NO;
@@ -998,6 +1035,8 @@ MHD_start_daemon (unsigned int options,
 }
 
 
+typedef void (*VfprintfFunctionPointerType)(void *, const char *, va_list);
+
 /**
  * Start a webserver on the given port.
  *
@@ -1034,7 +1073,7 @@ MHD_start_daemon_va (unsigned int options,
   if (retVal == NULL)
     return NULL;
   memset (retVal, 0, sizeof (struct MHD_Daemon));
-  retVal->options = options;
+  retVal->options = (enum MHD_OPTION)options;
   retVal->port = port;
   retVal->apc = apc;
   retVal->apc_cls = apc_cls;
@@ -1052,9 +1091,21 @@ MHD_start_daemon_va (unsigned int options,
   if (options & MHD_USE_SSL)
     {
       /* lock MHD_gnutls_global mutex since it uses reference counting */
-      pthread_mutex_lock (&MHD_gnutls_init_mutex);
+      if (0 != pthread_mutex_lock (&MHD_gnutls_init_mutex))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (retVal, "Failed to aquire gnutls mutex\n");
+#endif
+	  abort();
+	}
       MHD__gnutls_global_init ();
-      pthread_mutex_unlock (&MHD_gnutls_init_mutex);
+      if (0 != pthread_mutex_unlock (&MHD_gnutls_init_mutex))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (retVal, "Failed to release gnutls mutex\n");
+#endif
+	  abort();
+	}
       /* set default priorities */
       MHD_tls_set_default_priority (&retVal->priority_cache, "", NULL);
       retVal->cred_type = MHD_GNUTLS_CRD_CERTIFICATE;
@@ -1112,13 +1163,13 @@ MHD_start_daemon_va (unsigned int options,
         case MHD_OPTION_EXTERNAL_LOGGER:
 #if HAVE_MESSAGES
           retVal->custom_error_log =
-            va_arg (ap, void (*)(void *cls, const char *, va_list));
+            va_arg (ap, VfprintfFunctionPointerType);
           retVal->custom_error_log_cls = va_arg (ap, void *);
 #else
-          va_arg (ap, void (*)(void *cls, const char *, ...));
+          va_arg (ap, VfprintfFunctionPointerType);
           va_arg (ap, void *);
 #endif
-	  break;
+          break;
         default:
 #if HAVE_MESSAGES
           if ((opt >= MHD_OPTION_HTTPS_MEM_KEY) &&
@@ -1144,26 +1195,37 @@ MHD_start_daemon_va (unsigned int options,
       && (retVal->worker_pool_size > 0))
     {
 #if HAVE_MESSAGES
-      FPRINTF (stderr,
+      fprintf (stderr,
                "MHD thread pooling only works with MHD_USE_SELECT_INTERNALLY\n");
 #endif
       free (retVal);
       return NULL;
     }
 
+#ifdef __SYMBIAN32__
+  if (0 != (options & (MHD_USE_SELECT_INTERNALLY | MHD_USE_THREAD_PER_CONNECTION)))
+    {
+#if HAVE_MESSAGES
+      fprintf (stderr,
+               "Threaded operations are not supported on Symbian.\n");
+#endif
+      free (retVal);
+      return NULL;
+    }
+#endif
   if ((options & MHD_USE_IPv6) != 0)
 #if HAVE_INET6
     socket_fd = SOCKET (PF_INET6, SOCK_STREAM, 0);
 #else
     {
 #if HAVE_MESSAGES
-      FPRINTF (stderr, "AF_INET6 not supported\n");
+      fprintf (stderr, "AF_INET6 not supported\n");
 #endif
       return NULL;
     }
 #endif
   else
-    socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);
+    socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);  
   if (socket_fd < 0)
     {
 #if HAVE_MESSAGES
@@ -1173,6 +1235,18 @@ MHD_start_daemon_va (unsigned int options,
       free (retVal);
       return NULL;
     }
+  if (socket_fd >= FD_SETSIZE)
+    {
+#if HAVE_MESSAGES
+      if ((options & MHD_USE_DEBUG) != 0)
+        FPRINTF (stderr,
+                 "Socket descriptor larger than FD_SETSIZE: %d > %d\n",
+                 socket_fd,
+                 FD_SETSIZE);
+#endif     
+      CLOSE (socket_fd);
+      return NULL;
+    }
   if ((SETSOCKOPT (socket_fd,
                    SOL_SOCKET,
                    SO_REUSEADDR,
@@ -1222,7 +1296,6 @@ MHD_start_daemon_va (unsigned int options,
       return NULL;
     }
 
-
   if (LISTEN (socket_fd, 20) < 0)
     {
 #if HAVE_MESSAGES
@@ -1316,7 +1389,7 @@ MHD_start_daemon_va (unsigned int options,
       retVal->worker_pool = malloc (sizeof (struct MHD_Daemon)
                                     * retVal->worker_pool_size);
       if (NULL == retVal->worker_pool)
-	goto thread_failed;
+        goto thread_failed;
 
       /* Start the workers in the pool */
       for (i = 0; i < retVal->worker_pool_size; ++i)
@@ -1326,8 +1399,8 @@ MHD_start_daemon_va (unsigned int options,
           memcpy (d, retVal, sizeof (struct MHD_Daemon));
 
           /* Adjust pooling params for worker daemons; note that memcpy()
-	     has already copied MHD_USE_SELECT_INTERNALLY thread model into
-	     the worker threads. */
+             has already copied MHD_USE_SELECT_INTERNALLY thread model into
+             the worker threads. */
           d->master = retVal;
           d->worker_pool_size = 0;
           d->worker_pool = NULL;
@@ -1350,7 +1423,7 @@ MHD_start_daemon_va (unsigned int options,
                * all previously-created workers. */
               goto thread_failed;
             }
-	}
+        }
     }
   return retVal;
 
@@ -1364,11 +1437,11 @@ thread_failed:
       CLOSE (socket_fd);
       pthread_mutex_destroy (&retVal->per_ip_connection_mutex);
       if (NULL != retVal->worker_pool)
-	free (retVal->worker_pool);
+        free (retVal->worker_pool);
       free (retVal);
       return NULL;
     }
-  
+
   /* Shutdown worker threads we've already created. Pretend
      as though we had fully initialized our daemon, but
      with a smaller number of threads than had been
@@ -1409,6 +1482,7 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
   void *unused;
   int fd;
   unsigned int i;
+  int rc;
 
   if (daemon == NULL)
     return;
@@ -1425,7 +1499,7 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 
 #if OSX
   /* without this, either (thread pool = 0) threads would get stuck or
-   * CLOSE would get stuck if attempted before (thread pool > 0) 
+   * CLOSE would get stuck if attempted before (thread pool > 0)
    * threads have ended */
   SHUTDOWN (fd, SHUT_RDWR);
 #else
@@ -1442,7 +1516,14 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
     pthread_kill (daemon->worker_pool[i].pid, SIGALRM);
   for (i = 0; i < daemon->worker_pool_size; ++i)
     {
-      pthread_join (daemon->worker_pool[i].pid, &unused);
+      if (0 != (rc = pthread_join (daemon->worker_pool[i].pid, &unused)))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (daemon, "Failed to join a thread: %s\n",
+		    STRERROR (rc));
+#endif
+	  abort();		
+	}
       MHD_close_connections (&daemon->worker_pool[i]);
     }
   free (daemon->worker_pool);
@@ -1452,7 +1533,14 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
         && (0 == daemon->worker_pool_size)))
     {
       pthread_kill (daemon->pid, SIGALRM);
-      pthread_join (daemon->pid, &unused);
+      if (0 != (rc = pthread_join (daemon->pid, &unused)))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (daemon, "Failed to join a thread: %s\n",
+		    STRERROR (rc));
+#endif
+	  abort();
+	}
     }
   MHD_close_connections (daemon);
 
@@ -1473,9 +1561,21 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
       if (daemon->x509_cred)
         MHD__gnutls_certificate_free_credentials (daemon->x509_cred);
       /* lock MHD_gnutls_global mutex since it uses reference counting */
-      pthread_mutex_lock (&MHD_gnutls_init_mutex);
+      if (0 != pthread_mutex_lock (&MHD_gnutls_init_mutex))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (daemon, "Failed to aquire gnutls mutex\n");
+#endif
+	  abort();
+	}
       MHD__gnutls_global_deinit ();
-      pthread_mutex_unlock (&MHD_gnutls_init_mutex);
+      if (0 != pthread_mutex_unlock (&MHD_gnutls_init_mutex))
+	{
+#if HAVE_MESSAGES
+	  MHD_DLOG (daemon, "Failed to release gnutls mutex\n");
+#endif
+	  abort();
+	}
     }
 #endif
   pthread_mutex_destroy (&daemon->per_ip_connection_mutex);
@@ -1496,12 +1596,18 @@ const union MHD_DaemonInfo *
 MHD_get_daemon_info (struct MHD_Daemon *daemon,
                      enum MHD_DaemonInfoType infoType, ...)
 {
-  return NULL;
+  switch (infoType)
+    {
+    case MHD_DAEMON_INFO_LISTEN_FD:
+      return (const union MHD_DaemonInfo *) &daemon->socket_fd;
+   default:
+      return NULL;
+    };
 }
 
 /**
  * Obtain the version of this library
- * 
+ *
  * @return static version string, e.g. "0.4.1"
  */
 const char *
@@ -1522,11 +1628,19 @@ sigalrmHandler (int sig)
 }
 #endif
 
+#ifdef __GNUC__
+#define ATTRIBUTE_CONSTRUCTOR __attribute__ ((constructor))
+#define ATTRIBUTE_DESTRUCTOR __attribute__ ((destructor))
+#else  // !__GNUC__
+#define ATTRIBUTE_CONSTRUCTOR
+#define ATTRIBUTE_DESTRUCTOR
+#endif  // __GNUC__
+
 /**
  * Initialize the signal handler for SIGALRM
  * and do other setup work.
  */
-void __attribute__ ((constructor)) MHD_init ()
+void ATTRIBUTE_CONSTRUCTOR MHD_init ()
 {
 #ifndef WINDOWS
   /* make sure SIGALRM does not kill us */
@@ -1544,7 +1658,7 @@ void __attribute__ ((constructor)) MHD_init ()
 #endif
 }
 
-void __attribute__ ((destructor)) MHD_fini ()
+void ATTRIBUTE_DESTRUCTOR MHD_fini ()
 {
 #if HTTPS_SUPPORT
   if (0 != pthread_mutex_destroy(&MHD_gnutls_init_mutex))
