@@ -81,17 +81,21 @@ extern "C"
 /**
  * Current version of the library.
  */
-#define MHD_VERSION 0x00040200
+#define MHD_VERSION 0x00040400
 
 /**
- * MHD-internal return codes.
+ * MHD-internal return code for "YES".
  */
 #define MHD_YES 1
 
+/**
+ * MHD-internal return code for "NO".
+ */
 #define MHD_NO 0
 
 /**
- * Constant used to indicate unknown size.
+ * Constant used to indicate unknown size (use when
+ * creating a response).
  */
 #define MHD_SIZE_UNKNOWN  ((uint64_t) -1LL)
 
@@ -174,6 +178,7 @@ extern "C"
 #define MHD_HTTP_HEADER_CONTENT_MD5 "Content-MD5"
 #define MHD_HTTP_HEADER_CONTENT_RANGE "Content-Range"
 #define MHD_HTTP_HEADER_CONTENT_TYPE "Content-Type"
+#define MHD_HTTP_HEADER_COOKIE "Cookie"
 #define MHD_HTTP_HEADER_DATE "Date"
 #define MHD_HTTP_HEADER_ETAG "ETag"
 #define MHD_HTTP_HEADER_EXPECT "Expect"
@@ -195,6 +200,8 @@ extern "C"
 #define MHD_HTTP_HEADER_REFERER "Referer"
 #define MHD_HTTP_HEADER_RETRY_AFTER "Retry-After"
 #define MHD_HTTP_HEADER_SERVER "Server"
+#define MHD_HTTP_HEADER_SET_COOKIE "Set-Cookie"
+#define MHD_HTTP_HEADER_SET_COOKIE2 "Set-Cookie2"
 #define MHD_HTTP_HEADER_TE "TE"
 #define MHD_HTTP_HEADER_TRAILER "Trailer"
 #define MHD_HTTP_HEADER_TRANSFER_ENCODING "Transfer-Encoding"
@@ -595,7 +602,13 @@ enum MHD_DaemonInfoType
    * algorithm should be passed as an extra
    * argument (of type 'enum MHD_GNUTLS_HashAlgorithm').
    */
-  MHD_DAEMON_INFO_MAC_KEY_SIZE
+  MHD_DAEMON_INFO_MAC_KEY_SIZE,
+
+  /**
+   * Request the file descriptor for the listening socket.
+   * No extra arguments should be passed.
+   */
+  MHD_DAEMON_INFO_LISTEN_FD
 };
 
 
@@ -1176,6 +1189,11 @@ union MHD_DaemonInfo
    * Size of the mac key (unit??)
    */
   size_t mac_key_size;
+
+  /**
+   * Listen socket file descriptor
+   */
+  int listen_fd;
 };
 
 /**
